@@ -15,21 +15,23 @@
 import functools
 import uuid
 
+from oslo_config import cfg
+from oslo_log import log
+from oslo_utils import strutils
+import six
+
 from keystone.common import authorization
 from keystone.common import dependency
 from keystone.common import driver_hints
 from keystone.common import utils
 from keystone.common import wsgi
-from keystone import config
 from keystone import exception
-from keystone.i18n import _
+from keystone.i18n import _, _LW
 from keystone.models import token_model
-from keystone.openstack.common import log
-from keystone.openstack.common import strutils
 
 
 LOG = log.getLogger(__name__)
-CONF = config.CONF
+CONF = cfg.CONF
 
 
 def v2_deprecated(f):
@@ -78,7 +80,7 @@ def _build_policy_check_credentials(self, action, context, kwargs):
         # backing store.
         wsgi.validate_token_bind(context, token_ref)
     except exception.TokenNotFound:
-        LOG.warning(_('RBAC: Invalid token'))
+        LOG.warning(_LW('RBAC: Invalid token'))
         raise exception.Unauthorized()
 
     auth_context = authorization.token_to_auth_context(token_ref)
@@ -103,7 +105,7 @@ def protected(callback=None):
         @functools.wraps(f)
         def inner(self, context, *args, **kwargs):
             if 'is_admin' in context and context['is_admin']:
-                LOG.warning(_('RBAC: Bypassing authorization'))
+                LOG.warning(_LW('RBAC: Bypassing authorization'))
             elif callback is not None:
                 prep_info = {'f_name': f.__name__,
                              'input_attr': kwargs}
@@ -200,7 +202,7 @@ def filterprotected(*filters):
 
                 LOG.debug('RBAC: Authorization granted')
             else:
-                LOG.warning(_('RBAC: Bypassing authorization'))
+                LOG.warning(_LW('RBAC: Bypassing authorization'))
             return f(self, context, filters, **kwargs)
         return wrapper
     return _filterprotected
@@ -224,6 +226,28 @@ class V2Controller(wsgi.Application):
         ref.pop('domain_id', None)
         return ref
 
+    @staticmethod
+    def filter_domain(ref):
+        """Remove domain since v2 calls are not domain-aware.
+
+        V3 Fernet tokens builds the users with a domain in the token data.
+        This method will ensure that users create in v3 belong to the default
+        domain.
+
+        """
+        if 'domain' in ref:
+            if ref['domain'].get('id') != CONF.identity.default_domain_id:
+                raise exception.Unauthorized(
+                    _('Non-default domain is not supported'))
+            del ref['domain']
+        return ref
+
+    @staticmethod
+    def filter_project_parent_id(ref):
+        """Remove parent_id since v2 calls are not hierarchy-aware."""
+        ref.pop('parent_id', None)
+        return ref
+
     @staticmethod
     def normalize_username_in_response(ref):
         """Adds username to outgoing user refs to match the v2 spec.
@@ -278,6 +302,7 @@ class V2Controller(wsgi.Application):
         def _normalize_and_filter_user_properties(ref):
             """Run through the various filter/normalization methods."""
             _format_default_project_id(ref)
+            V2Controller.filter_domain(ref)
             V2Controller.filter_domain_id(ref)
             V2Controller.normalize_username_in_response(ref)
             return ref
@@ -289,6 +314,69 @@ class V2Controller(wsgi.Application):
         else:
             raise ValueError(_('Expected dict or list: %s') % type(ref))
 
+    @staticmethod
+    def v3_to_v2_project(ref):
+        """Convert a project_ref from v3 to v2.
+
+        * v2.0 projects are not domain aware, and should have domain_id removed
+        * v2.0 projects are not hierarchy aware, and should have parent_id
+          removed
+
+        This method should only be applied to project_refs being returned from
+        the v2.0 controller(s).
+
+        If ref is a list type, we will iterate through each element and do the
+        conversion.
+        """
+
+        def _filter_project_properties(ref):
+            """Run through the various filter methods."""
+            V2Controller.filter_domain_id(ref)
+            V2Controller.filter_project_parent_id(ref)
+            return ref
+
+        if isinstance(ref, dict):
+            return _filter_project_properties(ref)
+        elif isinstance(ref, list):
+            return [_filter_project_properties(x) for x in ref]
+        else:
+            raise ValueError(_('Expected dict or list: %s') % type(ref))
+
+    def format_project_list(self, tenant_refs, **kwargs):
+        """Format a v2 style project list, including marker/limits."""
+        marker = kwargs.get('marker')
+        first_index = 0
+        if marker is not None:
+            for (marker_index, tenant) in enumerate(tenant_refs):
+                if tenant['id'] == marker:
+                    # we start pagination after the marker
+                    first_index = marker_index + 1
+                    break
+            else:
+                msg = _('Marker could not be found')
+                raise exception.ValidationError(message=msg)
+
+        limit = kwargs.get('limit')
+        last_index = None
+        if limit is not None:
+            try:
+                limit = int(limit)
+                if limit < 0:
+                    raise AssertionError()
+            except (ValueError, AssertionError):
+                msg = _('Invalid limit value')
+                raise exception.ValidationError(message=msg)
+            last_index = first_index + limit
+
+        tenant_refs = tenant_refs[first_index:last_index]
+
+        for x in tenant_refs:
+            if 'enabled' not in x:
+                x['enabled'] = True
+        o = {'tenants': tenant_refs,
+             'tenants_links': []}
+        return o
+
 
 @dependency.requires('policy_api', 'token_provider_api')
 class V3Controller(wsgi.Application):
@@ -320,6 +408,12 @@ class V3Controller(wsgi.Application):
 
         return '%s/%s/%s' % (endpoint, 'v3', path.lstrip('/'))
 
+    def get_auth_context(self, context):
+        # TODO(dolphm): this method of accessing the auth context is terrible,
+        # but context needs to be refactored to always have reasonable values.
+        env_context = context.get('environment', {})
+        return env_context.get(authorization.AUTH_CONTEXT_ENV, {})
+
     @classmethod
     def full_url(cls, context, path=None):
         url = cls.base_url(context, path)
@@ -328,6 +422,27 @@ class V3Controller(wsgi.Application):
 
         return url
 
+    @classmethod
+    def query_filter_is_true(cls, filter_value):
+        """Determine if bool query param is 'True'.
+
+        We treat this the same way as we do for policy
+        enforcement:
+
+        {bool_param}=0 is treated as False
+
+        Any other value is considered to be equivalent to
+        True, including the absence of a value
+
+        """
+
+        if (isinstance(filter_value, six.string_types) and
+                filter_value == '0'):
+            val = False
+        else:
+            val = True
+        return val
+
     @classmethod
     def _add_self_referential_link(cls, context, ref):
         ref.setdefault('links', {})
@@ -589,15 +704,15 @@ class V3Controller(wsgi.Application):
                 _('domain_id is required as part of entity'))
         except (exception.TokenNotFound,
                 exception.UnsupportedTokenVersionException):
-            LOG.warning(_('Invalid token found while getting domain ID '
-                          'for list request'))
+            LOG.warning(_LW('Invalid token found while getting domain ID '
+                            'for list request'))
             raise exception.Unauthorized()
 
         if token_ref.domain_scoped:
             return token_ref.domain_id
         else:
             LOG.warning(
-                _('No domain information specified as part of list request'))
+                _LW('No domain information specified as part of list request'))
             raise exception.Unauthorized()
 
     def _get_domain_id_from_token(self, context):
@@ -624,8 +739,8 @@ class V3Controller(wsgi.Application):
                 _('A domain-scoped token must be used'))
         except (exception.TokenNotFound,
                 exception.UnsupportedTokenVersionException):
-            LOG.warning(_('Invalid token found while getting domain ID '
-                          'for list request'))
+            LOG.warning(_LW('Invalid token found while getting domain ID '
+                            'for list request'))
             raise exception.Unauthorized()
 
         if token_ref.domain_scoped:
@@ -660,7 +775,7 @@ class V3Controller(wsgi.Application):
 
          """
         if 'is_admin' in context and context['is_admin']:
-            LOG.warning(_('RBAC: Bypassing authorization'))
+            LOG.warning(_LW('RBAC: Bypassing authorization'))
         else:
             action = 'identity:%s' % prep_info['f_name']
             # TODO(henry-nash) need to log the target attributes as well
