@@ -14,19 +14,21 @@
 
 """Workflow Logic the Identity service."""
 
+from oslo_config import cfg
+from oslo_log import log
+
 from keystone.common import controller
 from keystone.common import dependency
-from keystone import config
 from keystone import exception
-from keystone.i18n import _
-from keystone.openstack.common import log
+from keystone.i18n import _, _LW
+from keystone import notifications
 
 
-CONF = config.CONF
+CONF = cfg.CONF
 LOG = log.getLogger(__name__)
 
 
-@dependency.requires('assignment_api', 'identity_api')
+@dependency.requires('assignment_api', 'identity_api', 'resource_api')
 class User(controller.V2Controller):
 
     @controller.v2_deprecated
@@ -73,7 +75,7 @@ class User(controller.V2Controller):
         default_project_id = user.pop('tenantId', None)
         if default_project_id is not None:
             # Check to see if the project is valid before moving on.
-            self.assignment_api.get_project(default_project_id)
+            self.resource_api.get_project(default_project_id)
             user['default_project_id'] = default_project_id
 
         # The manager layer will generate the unique ID for users
@@ -114,7 +116,7 @@ class User(controller.V2Controller):
                 default_project_id is not None)):
             # Make sure the new project actually exists before we perform the
             # user update.
-            self.assignment_api.get_project(default_project_id)
+            self.resource_api.get_project(default_project_id)
 
         user_ref = self.v3_to_v2_user(
             self.identity_api.update_user(user_id, user))
@@ -125,7 +127,7 @@ class User(controller.V2Controller):
             if user_ref['tenantId'] != old_user_ref.get('tenantId'):
                 if old_user_ref.get('tenantId'):
                     try:
-                        member_role_id = config.CONF.member_role_id
+                        member_role_id = CONF.member_role_id
                         self.assignment_api.remove_role_from_user_and_project(
                             user_id, old_user_ref['tenantId'], member_role_id)
                     except exception.NotFound:
@@ -134,8 +136,8 @@ class User(controller.V2Controller):
                         # old tenant.  This could occur if roles aren't found
                         # or if the project is invalid or if there are no roles
                         # for the user on that project.
-                        msg = _('Unable to remove user %(user)s from '
-                                '%(tenant)s.')
+                        msg = _LW('Unable to remove user %(user)s from '
+                                  '%(tenant)s.')
                         LOG.warning(msg, {'user': user_id,
                                           'tenant': old_user_ref['tenantId']})
 
@@ -153,7 +155,7 @@ class User(controller.V2Controller):
                         # that the project is invalid or roles are some how
                         # incorrect.  This shouldn't prevent the return of the
                         # new ref.
-                        msg = _('Unable to add user %(user)s to %(tenant)s.')
+                        msg = _LW('Unable to add user %(user)s to %(tenant)s.')
                         LOG.warning(msg, {'user': user_id,
                                           'tenant': user_ref['tenantId']})
 
@@ -209,7 +211,8 @@ class UserV3(controller.V3Controller):
         # The manager layer will generate the unique ID for users
         ref = self._normalize_dict(user)
         ref = self._normalize_domain_id(context, ref)
-        ref = self.identity_api.create_user(ref)
+        initiator = notifications._get_request_audit_info(context)
+        ref = self.identity_api.create_user(ref, initiator)
         return UserV3.wrap_member(context, ref)
 
     @controller.filterprotected('domain_id', 'enabled', 'name')
@@ -235,7 +238,8 @@ class UserV3(controller.V3Controller):
         self._require_matching_id(user_id, user)
         self._require_matching_domain_id(
             user_id, user, self.identity_api.get_user)
-        ref = self.identity_api.update_user(user_id, user)
+        initiator = notifications._get_request_audit_info(context)
+        ref = self.identity_api.update_user(user_id, user, initiator)
         return UserV3.wrap_member(context, ref)
 
     @controller.protected()
@@ -256,7 +260,8 @@ class UserV3(controller.V3Controller):
 
     @controller.protected()
     def delete_user(self, context, user_id):
-        return self.identity_api.delete_user(user_id)
+        initiator = notifications._get_request_audit_info(context)
+        return self.identity_api.delete_user(user_id, initiator)
 
     @controller.protected()
     def change_password(self, context, user_id, user):
@@ -292,7 +297,8 @@ class GroupV3(controller.V3Controller):
         # The manager layer will generate the unique ID for groups
         ref = self._normalize_dict(group)
         ref = self._normalize_domain_id(context, ref)
-        ref = self.identity_api.create_group(ref)
+        initiator = notifications._get_request_audit_info(context)
+        ref = self.identity_api.create_group(ref, initiator)
         return GroupV3.wrap_member(context, ref)
 
     @controller.filterprotected('domain_id', 'name')
@@ -319,9 +325,11 @@ class GroupV3(controller.V3Controller):
         self._require_matching_id(group_id, group)
         self._require_matching_domain_id(
             group_id, group, self.identity_api.get_group)
-        ref = self.identity_api.update_group(group_id, group)
+        initiator = notifications._get_request_audit_info(context)
+        ref = self.identity_api.update_group(group_id, group, initiator)
         return GroupV3.wrap_member(context, ref)
 
     @controller.protected()
     def delete_group(self, context, group_id):
-        self.identity_api.delete_group(group_id)
+        initiator = notifications._get_request_audit_info(context)
+        self.identity_api.delete_group(group_id, initiator)
